package com.skybeacon.sdk.config;

import com.skybeacon.sdk.OADProcessCallback;
import com.skybeacon.sdk.locate.PublicStaticValues;
import com.skybeacon.sdk.locate.SKYBeacon;

import android.annotation.SuppressLint;
import android.content.Context;

/**
 * SeekcyBeacon升级类：连接、升级
 * 
 * @author frank
 * 
 */
@SuppressLint("HandlerLeak")
public class SKYBeaconOAD {

	public class HandleValue {
		public static final int CONNECT_SUCCESS = 1;
		public static final int CONNECT_FAILED = 2;
		public static final int MULTI_ID_BEACON = 3;
		public static final int SINGLE_ID_BEACON = 4;
		public static final int CONFIG_SUCCESS = 5;
		public static final int CONFIG_FAILED = 6;
		public static final int DISCONNECT = 7;
	}

	private Context context;
	private Object lock = new Object();
	private boolean isCommunicateFlag = false;
	private boolean isConnected = false;
	private SKYBeaconOADProcess skyBeaconOADProcess;
	private OADProcessCallback oadProcessCallback;

	/**
	 * 构造函数
	 * 
	 * @param context
	 */
	public SKYBeaconOAD(Context context) {
		super();
		this.context = context;
		PublicStaticValues.init(context);
	}

	/**
	 * 连接设备，只支持单设备连接
	 * 
	 * @param skyBeacon
	 * @param connectionStateCallback
	 * @return result
	 */
	public boolean setBeaconMode(final SKYBeacon skyBeacon, final String key) {
		skyBeaconOADProcess = new SKYBeaconOADProcess(context);
		boolean result = true;
		synchronized (lock) {
			if (isCommunicateFlag) {
				result = false;
			} else {
				Thread thread = new Thread(new Runnable() {
					@Override
					public void run() {
						// TODO Auto-generated method stub
						isCommunicateFlag = true;
						int ret = skyBeaconOADProcess.setImageBMode(skyBeacon, key);
						if (ret != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
							oadProcessCallback.onImageSwitch(true, false, ret, SKYBeaconConfigException.getMessage(ret));
						} else {
							oadProcessCallback.onImageSwitch(false, true, ret, SKYBeaconConfigException.getMessage(ret));
						}
						isCommunicateFlag = false;
					}
				});
				thread.start();
			}
		}

		return result;
	}

	public boolean setOADMode(final SKYBeacon skyBeacon, final String key) {
		skyBeaconOADProcess = new SKYBeaconOADProcess(context);
		boolean result = true;
		synchronized (lock) {
			if (isCommunicateFlag) {
				result = false;
			} else {
				Thread thread = new Thread(new Runnable() {

					@Override
					public void run() {
						// TODO Auto-generated method stub
						isCommunicateFlag = true;
						int ret = skyBeaconOADProcess.setImageAMode(skyBeacon, key);
						if (ret != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
							oadProcessCallback.onImageSwitch(false, false, ret, SKYBeaconConfigException.getMessage(ret));
						} else {
							oadProcessCallback.onImageSwitch(true, true, ret, SKYBeaconConfigException.getMessage(ret));
						}
						isCommunicateFlag = false;
					}
				});
				thread.start();
			}
		}

		return result;
	}

	public void setOnOADProcessCallback(OADProcessCallback onOadProcessCallback) {
		this.oadProcessCallback = onOadProcessCallback;
	}

	public boolean oadProcess(final SKYBeacon skyBeacon, final String filePath) {
		skyBeaconOADProcess = new SKYBeaconOADProcess(context);
		boolean result = true;
		synchronized (lock) {
			if (isCommunicateFlag) {
				result = false;
			} else {
				Thread thread = new Thread(new Runnable() {

					@Override
					public void run() {
						isCommunicateFlag = true;
						skyBeaconOADProcess.setOnOADProcessCallback(oadProcessCallback);
						int ret = skyBeaconOADProcess.startOADProcess(skyBeacon, filePath);
						if (ret != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
							oadProcessCallback.onOADFailed(0, 0, ret, SKYBeaconConfigException.getMessage(ret));
						}
						isCommunicateFlag = false;
					}
				});
				thread.start();
			}
		}

		return result;
	}

	/**
	 * 断开连接
	 */
	public void disconnect() {
		isCommunicateFlag = false;
		if (isConnected) {
			isConnected = false;
		}
		skyBeaconOADProcess.interruptConfig();
	}
}
