package com.skybeacon.sdk.config;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;
import java.util.List;
import java.util.Timer;
import java.util.UUID;

import com.skybeacon.sdk.OADProcessCallback;
import com.skybeacon.sdk.entity.VersionAdaption;
import com.skybeacon.sdk.locate.SKYBeacon;
import com.skybeacon.sdk.utils.Conversion;
import com.skybeacon.sdk.utils.DatasCheck;
import com.skybeacon.sdk.utils.DefaultStaticValues;

import android.app.Activity;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

/**
 * SeekcyBeacon通信封装类
 * 
 * @author frank
 * 
 */
public class SKYBeaconOADProcess {

	/**
	 * 配置中每个步骤的超时时间
	 */
	private static final int CONFIG_STEP_TIMEOUT_MILLISECOND = 15000;
	/**
	 * 默认的key
	 */
	private static final byte[] DEFAULT_CONFIG_KEY_BYTES = new byte[] { (byte) 0xD3, (byte) 0xC5, (byte) 0xC5, (byte) 0xCB, (byte) 0xC3, (byte) 0xD9 };

	public static class GattAttributes {
		static public String SEEYCY_BEACON_COMMUNICATE_SERVICE = "0000fff0-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_1_READ_UUID = "0000fff1-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_2_READ_UUID = "0000fff2-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_3_READ_UUID = "0000fff3-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_4_READ_UUID = "0000fff4-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_5_READ_UUID = "0000fff5-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_6_READ_UUID = "0000fff6-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_7_READ_UUID = "0000fff7-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_9_READ_UUID = "0000fff9-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_10_READ_UUID = "0000fffa-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_12_READ_UUID = "0000fffc-0000-1000-8000-00805f9b34fb";
		static public String CHARACTERISTIC_14_READ_UUID = "0000fffe-0000-1000-8000-00805f9b34fb";
	}

	public class GattAttributesTIOAD {
		static public final String SEEKCY_BEACON_OAD_PROCESS_SERVICE = "F000FFC0-0451-4000-B000-000000000000";
		static public final String SEEYCY_BEACON_OAD_MODE_SERVICE = "F000FFD0-0451-4000-B000-000000000000";
		static public final String CHARACTERISTIC_1_READ_UUID = "F000FFD1-0451-4000-B000-000000000000";
	}

	public class ConfigReadStates {
		static public final int READ_STATE_INIT = -1;
		static public final int READ_STATE_FAILED = 11;
		static public final int READ_STATE_TIMEOUT = 12;
		static public final int READ_STATE_FALSE_PROTOCOL = 21;
		static public final int READ_STATE_EMPTY = 22;
		static public final int READ_STATE_RETURN_KEY_SUCCESS = 31;
		static public final int READ_STATE_RETURN_CONFIG_SUCCESS = 32;
		static public final int READ_STATE_RETURN_CONFIG_PAUSE = 33;
		static public final int READ_STATE_RETURN_CONFIG_PASSCODE_ERROR = 34;
		static public final int READ_STATE_RETURN_CONFIG_HEADER_ERROR = 35;
		static public final int READ_STATE_GATT_SERVICE_NOT_FOUND = 41;
		static public final int READ_STATE_CHARACTERISTIC_NOT_FOUND = 42;
		static public final int READ_STATE_CHARACTERISTIC_1_INIT = 101;
		static public final int READ_STATE_CHARACTERISTIC_2_INIT = 102;
		static public final int READ_STATE_CHARACTERISTIC_3_INIT = 103;
		static public final int READ_STATE_CHARACTERISTIC_4_INIT = 104;
		static public final int READ_STATE_CHARACTERISTIC_5_INIT = 105;
		static public final int READ_STATE_CHARACTERISTIC_6_INIT = 106;
		static public final int READ_STATE_CHARACTERISTIC_7_INIT = 107;
		static public final int READ_STATE_CHARACTERISTIC_9_INIT = 109;
		static public final int READ_STATE_CHARACTERISTIC_10_INIT = 110;
		static public final int READ_STATE_CHARACTERISTIC_12_INIT = 112;
		static public final int READ_STATE_CHARACTERISTIC_14_INIT = 114;
		static public final int READ_STATE_RESET_CHARACTERISTIC_1_INIT = 115;

		static public final int READ_STATE_CHARACTERISTIC_1_END = 201;
		static public final int READ_STATE_CHARACTERISTIC_2_END = 202;
		static public final int READ_STATE_CHARACTERISTIC_3_END = 203;
		static public final int READ_STATE_CHARACTERISTIC_4_END = 204;
		static public final int READ_STATE_CHARACTERISTIC_5_END = 205;
		static public final int READ_STATE_CHARACTERISTIC_6_END = 206;
		static public final int READ_STATE_CHARACTERISTIC_7_END = 207;
		static public final int READ_STATE_CHARACTERISTIC_9_END = 209;
		static public final int READ_STATE_CHARACTERISTIC_10_END = 210;
		static public final int READ_STATE_CHARACTERISTIC_12_END = 212;
		static public final int READ_STATE_CHARACTERISTIC_14_END = 214;
		static public final int READ_STATE_RESET_CHARACTERISTIC_1_END = 215;
	}

	private static final short OAD_CONN_INTERVAL = 12; // 15 milliseconds
	private static final short OAD_SUPERVISION_TIMEOUT = 50; // 500 milliseconds
	private static final int GATT_WRITE_TIMEOUT = 300; // Milliseconds

	private static final int FILE_BUFFER_SIZE = 0x40000;
	public static final String FW_CUSTOM_DIRECTORY = Environment.DIRECTORY_DOWNLOADS;
	private static final String FW_FILE_A = "SensorTagImgA.bin";
	private static final String FW_FILE_B = "SensorTagImgB.bin";

	private static final int OAD_BLOCK_SIZE = 16;
	private static final int HAL_FLASH_WORD_SIZE = 4;
	private static final int OAD_BUFFER_SIZE = 2 + OAD_BLOCK_SIZE;
	private static final int OAD_IMG_HDR_SIZE = 8;
	private static final long TIMER_INTERVAL = 1000;
	private static final int SEND_INTERVAL = 20; // Milliseconds (make sure this
													// is longer than the
													// connection interval)
	private static final int BLOCKS_PER_CONNECTION = 1;
	private static final String TAG = "SKYBeaconOADProcess";
	private Context context;
	private boolean isConfiging = false;

	private BleServiceConnection serviceConnection = null;
	private BluetoothLeService bluetoothLeService = null;
	private boolean isBind = false;
	private boolean isRegistered = false;

	private Object lockedWrite = new Object();
	private Object lockedRead = new Object();
	private boolean isDeviceConnected = false;
	private boolean isGattServicesDiscover = false;
	private int readState = ConfigReadStates.READ_STATE_INIT;
	private boolean writeState = false;
	private byte randomKey = 0x00;

	private byte[] readDatas = new byte[20];
	private int readDatasLength = 0;
	private SKYBeacon skyBeacon;

	private Handler handler;

	private final byte[] mFileBuffer = new byte[FILE_BUFFER_SIZE];
	private byte[] mOadBuffer = new byte[OAD_BUFFER_SIZE];
	private ImgHdr mFileImgHdr = new ImgHdr();
	private ImgHdr mTargImgHdr = new ImgHdr();
	private Timer mTimer = null;
	private ProgInfo mProgInfo = new ProgInfo();

	private BluetoothGattCharacteristic mCharIdentify = null;
	private BluetoothGattCharacteristic mCharBlock = null;
	private BluetoothGattCharacteristic mCharReturn = null;

	private String notifyUUID = "";
	private boolean mProgramming = false;
	private boolean mStopSuccess = true;
	private OADProcessCallback oadProcessCallback = null;

	/**
	 * 监听器接口
	 * 
	 * @author frank
	 * 
	 */
	public interface HandlerMessageListener {
		void handlerMessage(final Message msg);
	}

	private HandlerMessageListener handlerMessageListener = null;

	/**
	 * 添加配置监听器
	 * 
	 * @param handlerMessageListener
	 *            listener
	 */
	public void setOnHandlerMessageListener(HandlerMessageListener handlerMessageListener) {
		this.handlerMessageListener = handlerMessageListener;
	}

	/**
	 * 添加烧录过程回调
	 * 
	 * @param oadProcessCallback
	 */
	public void setOnOADProcessCallback(OADProcessCallback oadProcessCallback) {
		this.oadProcessCallback = oadProcessCallback;
	}

	/**
	 * 构造函数
	 * 
	 * @param context
	 */
	public SKYBeaconOADProcess(Context context) {
		this.context = context;
	}

	/**
	 * 构造函数
	 * 
	 * @param context
	 */
	public SKYBeaconOADProcess(Context context, Handler handler) {
		this.context = context;
		this.handler = handler;
	}

	/**
	 * 打断配置，在任何异常退出连接、配置情况下，需要调用此函数
	 */
	public void interruptConfig() {
		try {
			skyBeacon = null;
			if (isConfiging) {
				isConfiging = false;
				synchronized (lockedWrite) {
					lockedWrite.notifyAll();
				}
				synchronized (lockedRead) {
					lockedRead.notifyAll();
				}
				if (isRegistered) {
					context.unregisterReceiver(gattUpdateReceiver);
					isRegistered = false;
				}
				if (bluetoothLeService != null) {
					bluetoothLeService.disconnect();
					// bluetoothLeService.close();
				}
				serviceConnection.onServiceDisconnected(((Activity) context).getComponentName());
				if (isBind) {
					context.unbindService(serviceConnection);
				}
			}
		} catch (Exception e) {
			// TODO: handle exception
		}
	}

	/**
	 * 连接设备，将设备设置为OAD模式
	 * 
	 * @param skyBeacon
	 * @return 返回错误码
	 */
	public int setImageBMode(SKYBeacon skyBeacon, String key) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		this.skyBeacon = new SKYBeacon();
		this.skyBeacon = (SKYBeacon) skyBeacon.clone();
		// 初始化成员变量
		isConfiging = true;
		isDeviceConnected = false;
		isGattServicesDiscover = false;
		readState = ConfigReadStates.READ_STATE_INIT;
		writeState = false;
		randomKey = 0x00;
		// 连接设备
		int flag = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		try {
			flag = connectDevice(skyBeacon.getDeviceAddress());
			// 连接成功的情况下继续进行设置操作
			if (flag == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				Thread.sleep(200);
				// 读取版本号
				readState = ConfigReadStates.READ_STATE_RESET_CHARACTERISTIC_1_INIT;
				flag = readBle(UUID.fromString(GattAttributesTIOAD.SEEYCY_BEACON_OAD_MODE_SERVICE), UUID.fromString(GattAttributesTIOAD.CHARACTERISTIC_1_READ_UUID));
				synchronized (lockedRead) {
					try {
						lockedRead.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
					} catch (InterruptedException e) {
						e.printStackTrace();
					} catch (Exception e) {
						// TODO: handle exception
						e.printStackTrace();
					}
				}
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					interruptConfig();
					return result;
				}
				if (readState != ConfigReadStates.READ_STATE_RESET_CHARACTERISTIC_1_END) {
					result = ConfigInfoCode.ErrorCode.CODE_READ_VERSION_FAILED;
					interruptConfig();
					return result;
				} else {
					if (VersionAdaption.isS4LRRTC(readDatas[0], readDatas[1], readDatas[2])) {
						// 如果是imageA（OAD Mode）模式下，需要切换到imageB模式下
						if ((readDatas[3] & 0xFF) == 0x00) {
							// 确定配置过程中的密钥
							String keyDefault = "";
							String usedKeyThis = "";
							try {
								keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES, DefaultStaticValues.STRING_BYTES_CONVERT);
							} catch (UnsupportedEncodingException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
							if ((readDatas[4] & 0xFF) == 0x01 && key != null) {
								usedKeyThis = key;
							} else {
								usedKeyThis = keyDefault;
							}
							Thread.sleep(1000);
							flag = writeCharacteristic(UUID.fromString(GattAttributesTIOAD.SEEYCY_BEACON_OAD_MODE_SERVICE), UUID.fromString(GattAttributesTIOAD.CHARACTERISTIC_1_READ_UUID),
									getKeyDatas(usedKeyThis));
							Thread.sleep(1000);
						} else {
							result = ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_ALREADY_IN_MODE;
							interruptConfig();
							return result;
						}
					} else {
						result = ConfigInfoCode.ErrorCode.CODE_MODE_NONSUPPORT;
						interruptConfig();
						return result;
					}
				}
			} else {
				result = flag;
			}
		} catch (Exception e) {
			// TODO: handle exception
			flag = ConfigInfoCode.ErrorCode.CODE_CONNECT_FAILED;
			interruptConfig();
		}
		interruptConfig();
		return result;
	}

	/**
	 * 连接设备，将设备设置为OAD模式
	 * 
	 * @param skyBeacon
	 * @return 返回错误码
	 */
	public int setImageAMode(SKYBeacon skyBeacon, String key) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		this.skyBeacon = new SKYBeacon();
		this.skyBeacon = (SKYBeacon) skyBeacon.clone();
		// 初始化成员变量
		isConfiging = true;
		isDeviceConnected = false;
		isGattServicesDiscover = false;
		readState = ConfigReadStates.READ_STATE_INIT;
		writeState = false;
		randomKey = 0x00;
		// 连接设备
		int flag = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		try {
			flag = connectDevice(skyBeacon.getDeviceAddress());
			Thread.sleep(200);
			// 连接成功的情况下继续进行设置操作
			if (flag == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				// 读取版本号
				readState = ConfigReadStates.READ_STATE_RESET_CHARACTERISTIC_1_INIT;
				flag = readBle(UUID.fromString(GattAttributesTIOAD.SEEYCY_BEACON_OAD_MODE_SERVICE), UUID.fromString(GattAttributesTIOAD.CHARACTERISTIC_1_READ_UUID));
				synchronized (lockedRead) {
					try {
						lockedRead.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
					} catch (InterruptedException e) {
						e.printStackTrace();
					} catch (Exception e) {
						// TODO: handle exception
						e.printStackTrace();
					}
				}
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					interruptConfig();
					return result;
				}
				if (readState != ConfigReadStates.READ_STATE_RESET_CHARACTERISTIC_1_END) {
					result = ConfigInfoCode.ErrorCode.CODE_READ_VERSION_FAILED;
					interruptConfig();
					return result;
				} else {
					if (VersionAdaption.isS4LRRTC(readDatas[0], readDatas[1], readDatas[2])) {
						// 如果是imageB模式下，需要切换到imageA（OAD Mode）模式下
						if ((readDatas[3] & 0xFF) == 0x01) {
							// 确定配置过程中的密钥
							String keyDefault = "";
							String usedKeyThis = "";
							try {
								keyDefault = new String(DEFAULT_CONFIG_KEY_BYTES, DefaultStaticValues.STRING_BYTES_CONVERT);
							} catch (UnsupportedEncodingException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
							if ((readDatas[4] & 0xFF) == 0x01 && key != null) {
								usedKeyThis = key;
							} else {
								usedKeyThis = keyDefault;
							}
							Thread.sleep(1000);
							flag = writeCharacteristic(UUID.fromString(GattAttributesTIOAD.SEEYCY_BEACON_OAD_MODE_SERVICE), UUID.fromString(GattAttributesTIOAD.CHARACTERISTIC_1_READ_UUID),
									getKeyDatas(usedKeyThis));
							Thread.sleep(1000);
						} else {
							result = ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_ALREADY_IN_MODE;
							interruptConfig();
							return result;
						}
					} else {
						result = ConfigInfoCode.ErrorCode.CODE_MODE_NONSUPPORT;
						interruptConfig();
						return result;
					}
				}
			} else {
				result = flag;
			}
		} catch (Exception e) {
			// TODO: handle exception
			flag = ConfigInfoCode.ErrorCode.CODE_CONNECT_FAILED;
			interruptConfig();
		}
		interruptConfig();
		return result;
	}

	private byte[] getKeyDatas(String key) {
		byte[] result = new byte[6];
		try {
			if (key.length() < 6) {
				System.arraycopy(key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT), 0, result, 0, key.length());
			} else {
				System.arraycopy(key.getBytes(DefaultStaticValues.STRING_BYTES_CONVERT), 0, result, 0, 6);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * OAD过程
	 * 
	 * @param skyBeacon
	 * @param filePath
	 * @return
	 */
	public int startOADProcess(SKYBeacon skyBeacon, String filePath) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		this.skyBeacon = new SKYBeacon();
		this.skyBeacon = (SKYBeacon) skyBeacon.clone();
		// 初始化成员变量
		isConfiging = true;
		isDeviceConnected = false;
		isGattServicesDiscover = false;
		readState = ConfigReadStates.READ_STATE_INIT;
		writeState = false;
		randomKey = 0x00;
		// 连接设备
		int flag = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		try {
			flag = connectDevice(skyBeacon.getDeviceAddress());
			// 连接成功的情况下继续进行设置操作
			if (flag == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
				// 读取版本号
				readState = ConfigReadStates.READ_STATE_RESET_CHARACTERISTIC_1_INIT;
				flag = readBle(UUID.fromString(GattAttributesTIOAD.SEEYCY_BEACON_OAD_MODE_SERVICE), UUID.fromString(GattAttributesTIOAD.CHARACTERISTIC_1_READ_UUID));
				synchronized (lockedRead) {
					try {
						lockedRead.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
					} catch (InterruptedException e) {
						e.printStackTrace();
					} catch (Exception e) {
						// TODO: handle exception
						e.printStackTrace();
					}
				}
				if (flag != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
					result = flag;
					interruptConfig();
					if (oadProcessCallback != null) {
						oadProcessCallback.onOADFailed(0, 0, ConfigInfoCode.ErrorCode.CODE_READ_VERSION_FAILED, SKYBeaconConfigException.getMessage(ConfigInfoCode.ErrorCode.CODE_READ_VERSION_FAILED));
					}
					return result;
				}
				if (readState != ConfigReadStates.READ_STATE_RESET_CHARACTERISTIC_1_END) {
					result = ConfigInfoCode.ErrorCode.CODE_READ_VERSION_FAILED;
					interruptConfig();
					if (oadProcessCallback != null) {
						oadProcessCallback.onOADFailed(0, 0, ConfigInfoCode.ErrorCode.CODE_READ_VERSION_FAILED, SKYBeaconConfigException.getMessage(ConfigInfoCode.ErrorCode.CODE_READ_VERSION_FAILED));
					}
					return result;
				} else {
					if (VersionAdaption.isS4LRRTC(readDatas[0], readDatas[1], readDatas[2])) {
						// 如果是imageB模式下，需要切换到imageA（OAD Mode）模式下
						if ((readDatas[3] & 0xFF) == 0x01) {
							result = ConfigInfoCode.ErrorCode.CODE_NOT_IN_OAD_MODE;
							interruptConfig();
							if (oadProcessCallback != null) {
								oadProcessCallback.onOADFailed(0, 0, ConfigInfoCode.ErrorCode.CODE_NOT_IN_OAD_MODE, SKYBeaconConfigException.getMessage(ConfigInfoCode.ErrorCode.CODE_NOT_IN_OAD_MODE));
							}
							return result;
						} else {
							// 相关的service赋值
							result = updateCharacteristic();
							if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
								interruptConfig();
								if (oadProcessCallback != null) {
									oadProcessCallback.onOADFailed(0, 0, result, SKYBeaconConfigException.getMessage(result));
								}
								return result;
							}
							// 已经处于imageA模式下，开始升级过程
							// 第一步读取文件数据
							if (!loadFile(filePath, false)) {
								result = ConfigInfoCode.ErrorCode.CODE_READ_OAD_IMAGE_FAILED;
								interruptConfig();
								if (oadProcessCallback != null) {
									oadProcessCallback.onOADFailed(0, 0, ConfigInfoCode.ErrorCode.CODE_READ_OAD_IMAGE_FAILED, SKYBeaconConfigException.getMessage(ConfigInfoCode.ErrorCode.CODE_READ_OAD_IMAGE_FAILED));
								}
								return result;
							}
							// 第二步读取ImageA固件的版本信息等
							notifyUUID = "";
							result = bluetoothLeService.setCharacteristicNotification(mCharIdentify, true);
							if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
								interruptConfig();
								return result;
							}
							mCharIdentify.setValue(new byte[] {});
							boolean tmpRet = bluetoothLeService.writeCharacteristic(mCharIdentify);
							Thread.sleep(500);
							if (tmpRet && (notifyUUID.equals(mCharIdentify.getUuid().toString())) && (readDatas != null && readDatas.length >= 4)) {
								mTargImgHdr.ver = Conversion.buildUint16(readDatas[1], readDatas[0]);
								mTargImgHdr.imgType = ((mTargImgHdr.ver & 1) == 1) ? 'B' : 'A';
								mTargImgHdr.len = Conversion.buildUint16(readDatas[3], readDatas[2]);
								// // displayImageInfo(mTargImage, mTargImgHdr);
							} else {
								interruptConfig();
								if (oadProcessCallback != null) {
									oadProcessCallback.onOADFailed(0, 0, result, SKYBeaconConfigException.getMessage(result));
								}
								return result;
							}
							// Initialize stats
							mProgInfo.reset();
							// 判断是否符合升级条件（imgType判断）
							boolean ready = mFileImgHdr.imgType != mTargImgHdr.imgType;
							if (!ready) {
								result = ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_VERSION_FALSE;
								interruptConfig();
								if (oadProcessCallback != null) {
									oadProcessCallback.onOADFailed(0, 0, ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_VERSION_FALSE, SKYBeaconConfigException.getMessage(ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_VERSION_FALSE));
								}
								return result;
							}
							// Prepare data for request (try image A and B
							// respectively, only one of
							// them will give a notification with the image
							// info)
							byte[] buf = new byte[8];
							for (int i = 0; i < 8; i++) {
								buf[i] = mFileBuffer[i + 4];
							}
							// Send image notification
							notifyUUID = "";
							result = bluetoothLeService.setCharacteristicNotification(mCharBlock, true);
							if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
								interruptConfig();
								return result;
							}
							result = bluetoothLeService.setCharacteristicNotification(mCharReturn, true);
							if (result != ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
								interruptConfig();
								return result;
							}
							mCharIdentify.setValue(buf);
							tmpRet = bluetoothLeService.writeCharacteristic(mCharIdentify);
							Thread.sleep(500);
							if (tmpRet && (notifyUUID.equals(mCharBlock.getUuid().toString())) && (readDatas != null && readDatas.length >= 2)) {
								mProgramming = true;
								Log.d("FwUpdateActivity", String.format("NB: %02x%02x", readDatas[1], readDatas[0]));
								mCharBlock.setWriteType(BluetoothGattCharacteristic.WRITE_TYPE_NO_RESPONSE);
								bluetoothLeService.setCharacteristicNotification(mCharBlock, false);
								new Thread(new OadTask()).start();
							} else {
								result = ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_FIRST_WRITE_BLOCK_FAILED;
								interruptConfig();
								if (oadProcessCallback != null) {
									oadProcessCallback.onOADFailed((mFileImgHdr.len * 4), mProgInfo.iBytes, ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_FIRST_WRITE_BLOCK_FAILED, SKYBeaconConfigException.getMessage(ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_FIRST_WRITE_BLOCK_FAILED));
								}
								return result;
							}
						}
					} else {
						result = ConfigInfoCode.ErrorCode.CODE_MODE_NONSUPPORT;
						interruptConfig();
						return result;
					}
				}
			} else {
				result = flag;
				if (oadProcessCallback != null) {
					oadProcessCallback.onOADFailed(0, 0, flag, SKYBeaconConfigException.getMessage(flag));
				}
			}
		} catch (Exception e) {
			// TODO: handle exception
			flag = ConfigInfoCode.ErrorCode.CODE_CONNECT_FAILED;
			interruptConfig();
		}
		// interruptConfig();
		return result;
	}

	public int stopOADProcess() {
		int result = 0;
		mProgramming = false;
		int i = 0;
		while (mStopSuccess && i < 10) {
			try {
				Thread.sleep(200);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		if (!mStopSuccess) {
			result = -1;
		}
		return result;
	}

	public SKYBeacon getSkyBeacon() {
		return skyBeacon;
	}

	private class OadTask implements Runnable {
		@Override
		public void run() {
			mStopSuccess = false;
			while (mProgramming) {
				try {
					Thread.sleep(SEND_INTERVAL);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				for (int i = 0; i < BLOCKS_PER_CONNECTION & mProgramming; i++) {
					programBlock();
				}
			}
			if (!mProgramming) {
				mStopSuccess = true;
			}
		}
	}

	private void programBlock() {
		if (!mProgramming) {
			return;
		}

		if (mProgInfo.iBlocks < mProgInfo.nBlocks) {
			mProgramming = true;
			String msg = new String();

			// Prepare block
			mOadBuffer = new byte[OAD_BLOCK_SIZE + 2];
			mOadBuffer[0] = Conversion.loUint16(mProgInfo.iBlocks);
			mOadBuffer[1] = Conversion.hiUint16(mProgInfo.iBlocks);
			System.arraycopy(mFileBuffer, mProgInfo.iBytes, mOadBuffer, 2, OAD_BLOCK_SIZE);
			// Send block
			mCharBlock.setValue(mOadBuffer);
			Log.d("FwUpdateActivity", String.format("TX Block %02x%02x", mOadBuffer[1], mOadBuffer[0]));
			boolean success = bluetoothLeService.writeCharacteristic(mCharBlock);
			if (success) {
				oadProcessCallback.onProcessCallback((mFileImgHdr.len * 4), mProgInfo.iBytes, mProgInfo.iBytes * 100 / (mFileImgHdr.len * 4));
				// Update stats
				mProgInfo.iBlocks++;
				mProgInfo.iBytes += OAD_BLOCK_SIZE;
				if (mProgInfo.iBlocks == mProgInfo.nBlocks) {
					mProgramming = false;
					try {
						Thread.sleep(2000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			} else {
				mProgramming = false;
				msg = "GATT writeCharacteristic failed\n";
				Log.d("FwUpdateActivity", "GATT writeCharacteristic failed: " + String.format("TX Block %02x%02x", mOadBuffer[1], mOadBuffer[0]));
				oadProcessCallback.onOADFailed((mFileImgHdr.len * 4), mProgInfo.iBytes, ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_FIRST_WRITE_BLOCK_FAILED, SKYBeaconConfigException.getMessage(ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_FIRST_WRITE_BLOCK_FAILED));
			}
		} else {
			mProgramming = false;
			oadProcessCallback.onOADFailed((mFileImgHdr.len * 4), mProgInfo.iBytes, ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_FIRST_WRITE_BLOCK_FAILED, SKYBeaconConfigException.getMessage(ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_FIRST_WRITE_BLOCK_FAILED));
		}
	}

	// Handles various events fired by the Service.
	// ACTION_GATT_CONNECTED: connected to a GATT server.
	// ACTION_GATT_DISCONNECTED: disconnected from a GATT server.
	// ACTION_GATT_SERVICES_DISCOVERED: discovered GATT services.
	// ACTION_DATA_AVAILABLE: received data from the device. This can be
	// a
	// result of read
	// or notification operations.
	private final BroadcastReceiver gattUpdateReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			if (!isConfiging) {
				return;
			}
			final String action = intent.getAction();
			if (BluetoothLeService.ACTION_GATT_CONNECTED.equals(action)) {
				updateConnectionState(true);
				Log.i(TAG, "ble connected");
			} else if (BluetoothLeService.ACTION_GATT_DISCONNECTED.equals(action)) {
				updateConnectionState(false);
				Log.i(TAG, "ble disconnected");
			} else if (BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED.equals(action)) {
				bluetoothLeService = serviceConnection.getBleService();
				isGattServicesDiscover = true;
				synchronized (lockedRead) {
					lockedRead.notifyAll();
				}
				Log.i(TAG, "ble displayserice");
			} else if (BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED_FAILED.equals(action)) {
				isGattServicesDiscover = false;
				synchronized (lockedRead) {
					lockedRead.notifyAll();
				}
				Log.i(TAG, "ble displayserice failed");
			} else if (BluetoothLeService.ACTION_DATA_AVAILABLE.equals(action)) {
				String datas = intent.getStringExtra(BluetoothLeService.EXTRA_DATA);
				Log.i(TAG, "ble display data: " + datas);
				try {
					readDatas = datas.getBytes("ISO-8859-1");
					notifyUUID = intent.getStringExtra(BluetoothLeService.EXTRA_UUID);
					if (mCharReturn != null && mCharReturn.getUuid().toString().equals(notifyUUID)) {
						if (oadProcessCallback != null) {
							if (readDatas.length == 1 && readDatas[0] == 0x00) {
								oadProcessCallback.onOADSuccess((mFileImgHdr.len * 4), (mFileImgHdr.len * 4));
							} else {
								oadProcessCallback.onOADFailed((mFileImgHdr.len * 4), mProgInfo.iBytes, ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_FIRST_WRITE_BLOCK_FAILED, SKYBeaconConfigException.getMessage(ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_FIRST_WRITE_BLOCK_FAILED));
							}
						}
					}
				} catch (UnsupportedEncodingException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			} else if (BluetoothLeService.ACTION_DATA_WRITE_SUCCESS.equals(action)) {
				Log.i(TAG, "ble write data success");
				updateWriteDatas(true);
			} else if (BluetoothLeService.ACTION_DATA_WRITE_FAILED.equals(action)) {
				Log.i(TAG, "ble write data failed");
				updateWriteDatas(false);
			} else if (BluetoothLeService.ACTION_DATA_READ_SUCCESS.equals(action)) {
				String readDatas = intent.getStringExtra(BluetoothLeService.EXTRA_DATA);
				updateReadDatas(readDatas);
				Log.i(TAG, "ble read data success");
			} else if (BluetoothLeService.ACTION_DATA_READ_FAILED.equals(action)) {
				updateReadDatas("failed");
				Log.i(TAG, "ble read data failed");
			}
		}
	};

	private int updateCharacteristic() {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (bluetoothLeService != null) {
			List<BluetoothGattService> tmpGattService = bluetoothLeService.getBluetoothGatt().getServices();
			for (int i = 0; i < tmpGattService.size(); i++) {
				if (tmpGattService.get(i).getUuid().toString().toUpperCase().equals(GattAttributesTIOAD.SEEKCY_BEACON_OAD_PROCESS_SERVICE)) {
					List<BluetoothGattCharacteristic> charListOad = tmpGattService.get(i).getCharacteristics();
					if (charListOad.size() >= 4) {
						mCharIdentify = charListOad.get(0);
						mCharBlock = charListOad.get(1);
						mCharReturn = charListOad.get(3);
					} else {
						result = ConfigInfoCode.ErrorCode.CODE_OAD_IMAGE_SERVICE_FALSE;
					}
				}
			}
		}
		return result;
	}

	private void updateConnectionState(final boolean state) {
		isDeviceConnected = state;
		Thread thread = new Thread(new Runnable() {

			@Override
			public void run() {
				// TODO Auto-generated method stub
				if (state) {
					// isDeviceConnected = true;
				} else {
					// isDeviceConnected = false;
					synchronized (lockedRead) {
						lockedRead.notifyAll();
					}
					Message msg = new Message();
					msg.what = SKYBeaconCommunication.HandleValue.DISCONNECT;
					if (handler != null) {
						handler.sendMessage(msg);
					}
				}
			}
		});
		thread.start();
	}

	private void updateReadDatas(final String datas) {
		Thread thread = new Thread(new Runnable() {

			@Override
			public void run() {
				// TODO Auto-generated method stub
				if (datas != null && datas.equals("failed")) {
					// 读取失败
					readState = ConfigReadStates.READ_STATE_FAILED;
					// return;
				} else {
					byte[] readDatasTmp = null;
					try {
						readDatasTmp = datas.getBytes("ISO-8859-1");
					} catch (UnsupportedEncodingException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					if (readDatasTmp == null) {
						readState = ConfigReadStates.READ_STATE_EMPTY;
						return;
					}
					if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_1_INIT) {
						if (readDatasTmp.length == 2 && readDatasTmp[0] == 0x54) {
							readState = ConfigReadStates.READ_STATE_RETURN_KEY_SUCCESS;
							randomKey = readDatasTmp[1];
							// isRandomKeyEnable = true;
						} else if (readDatasTmp.length == 2 && readDatasTmp[0] == (byte) 0x86 && readDatasTmp[1] == (byte) 0x27) {
							readState = ConfigReadStates.READ_STATE_RETURN_CONFIG_SUCCESS;
						} else if (readDatasTmp.length == 2 && readDatasTmp[0] == (byte) 0x86 && readDatasTmp[1] == (byte) 0x28) {
							readState = ConfigReadStates.READ_STATE_RETURN_CONFIG_PAUSE;
						} else if (readDatasTmp.length == 2 && readDatasTmp[0] == (byte) 0x86 && readDatasTmp[1] == (byte) 0x29) {
							readState = ConfigReadStates.READ_STATE_RETURN_CONFIG_PASSCODE_ERROR;
						} else if (readDatasTmp.length == 2 && readDatasTmp[0] == (byte) 0x86 && readDatasTmp[1] == (byte) 0x2A) {
							readState = ConfigReadStates.READ_STATE_RETURN_CONFIG_HEADER_ERROR;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_2_INIT) {
						if (readDatas.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp, readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_2_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_3_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp, readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_3_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_4_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp, readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_4_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_5_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp, readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_5_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_6_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp, readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_6_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_7_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp, readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_7_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_9_INIT) {
						if (readDatasTmp.length == 1) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp, readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_9_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_10_INIT) {
						if (readDatasTmp.length == 6) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp, readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_10_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_CHARACTERISTIC_12_INIT) {
						if (readDatasTmp.length == 2) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp, readDatasLength);
							readState = ConfigReadStates.READ_STATE_CHARACTERISTIC_12_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else if (readState == ConfigReadStates.READ_STATE_RESET_CHARACTERISTIC_1_INIT) {
						if (readDatasTmp.length == 20) {
							readDatasLength = readDatasTmp.length;
							readDatas = Arrays.copyOf(readDatasTmp, readDatasLength);
							readState = ConfigReadStates.READ_STATE_RESET_CHARACTERISTIC_1_END;
						} else {
							readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
						}
					} else {
						readState = ConfigReadStates.READ_STATE_FALSE_PROTOCOL;
					}
				}
				synchronized (lockedRead) {
					lockedRead.notifyAll();
				}
			}
		});
		thread.start();
	}

	private void updateWriteDatas(final boolean isSuccess) {
		Thread thread = new Thread(new Runnable() {

			@Override
			public void run() {
				// TODO Auto-generated method stub
				if (isSuccess) {
					writeState = true;
				} else {
					writeState = false;
				}
				synchronized (lockedWrite) {
					lockedWrite.notifyAll();
				}
			}
		});
		thread.start();
	}

	/**
	 * 读取特定service的特定characteristic的数据
	 * 
	 * @param bluetoothLeService
	 * @param suuid
	 * @param cuuid
	 * @return
	 */
	private int readBle(UUID suuid, UUID cuuid) {
		int ret = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (bluetoothLeService != null) {
			BluetoothGattService tmpGattService = null;
			tmpGattService = bluetoothLeService.getBluetoothGatt().getService(suuid);
			if (tmpGattService != null) {
				BluetoothGattCharacteristic gattCharacteristics = null;
				gattCharacteristics = tmpGattService.getCharacteristic(cuuid);
				if (gattCharacteristics != null) {
					boolean retTmp = false;
					retTmp = bluetoothLeService.readCharacteristic(gattCharacteristics);
					if (!retTmp) {
						ret = ConfigInfoCode.ErrorCode.CODE_READ_CHARACTERISTIC_FAILED;
					}
				} else {
					ret = ConfigInfoCode.ErrorCode.CODE_CHARACTERISTIC_NOT_FOUND;
				}
			} else {
				ret = ConfigInfoCode.ErrorCode.CODE_GATT_SERVICE_NOT_FOUND;
			}
		}
		return ret;
	}

	/**
	 * 向特定的service的特定的characteristic写入数据
	 * 
	 * @param bluetoothLeService
	 * @param suuid
	 * @param cuuid
	 * @param writeDatas
	 * @return
	 */
	private int writeBle(UUID suuid, UUID cuuid, byte[] writeDatas) {
		int ret = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		if (bluetoothLeService != null) {
			BluetoothGattService tmpGattService = null;
			tmpGattService = bluetoothLeService.getBluetoothGatt().getService(suuid);
			if (tmpGattService != null) {
				BluetoothGattCharacteristic gattCharacteristics = null;
				gattCharacteristics = tmpGattService.getCharacteristic(cuuid);
				if (gattCharacteristics != null) {
					bluetoothLeService.setCharacteristicNotification(gattCharacteristics, true);
					gattCharacteristics.setValue(writeDatas);
					boolean retTmp = bluetoothLeService.writeCharacteristic(gattCharacteristics);
					if (!retTmp) {
						ret = ConfigInfoCode.ErrorCode.CODE_WRITE_CHARACTERISTIC_FAILED;
					}
				} else {
					ret = ConfigInfoCode.ErrorCode.CODE_CHARACTERISTIC_NOT_FOUND;
				}
			} else {
				ret = ConfigInfoCode.ErrorCode.CODE_GATT_SERVICE_NOT_FOUND;
			}
		}
		return ret;
	}

	/**
	 * 设备连接
	 * 
	 * @param skyBeacon
	 * @return
	 */
	private int connectDevice(String deviceAddress) {
		if (!DatasCheck.isMac(deviceAddress)) {
			return ConfigInfoCode.ErrorCode.CODE_NOT_SEEKCY_BEACON;
		}
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		isDeviceConnected = false;
		skyBeacon = new SKYBeacon(deviceAddress);
		serviceConnection = new BleServiceConnection(bluetoothLeService, deviceAddress);
		context.registerReceiver(gattUpdateReceiver, BluetoothLeService.makeGattUpdateIntentFilter());
		isRegistered = true;
		Intent gattServiceIntent = new Intent(context, BluetoothLeService.class);

		synchronized (lockedRead) {
			try {
				isBind = context.bindService(gattServiceIntent, serviceConnection, Context.BIND_AUTO_CREATE);
				lockedRead.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
			}
		}
		if (!isDeviceConnected) {
			result = ConfigInfoCode.ErrorCode.CODE_CONNECT_FAILED;
		} else if (isDeviceConnected && !isGattServicesDiscover) {
			result = ConfigInfoCode.ErrorCode.CODE_DISCOVER_GATT_SERVICE_FAILED;
		}
		return result;
	}

	private int getReadStateStart(String gattAttributes) {
		int ret = ConfigReadStates.READ_STATE_INIT;
		if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_1_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_1_INIT;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_2_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_2_INIT;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_3_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_3_INIT;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_4_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_4_INIT;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_5_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_5_INIT;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_6_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_6_INIT;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_7_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_7_INIT;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_9_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_9_INIT;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_10_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_10_INIT;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_12_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_12_INIT;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_14_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_14_INIT;
		} else if (gattAttributes.equals(GattAttributesTIOAD.CHARACTERISTIC_1_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_RESET_CHARACTERISTIC_1_INIT;
		}
		return ret;
	}

	private int getReadStateEnd(String gattAttributes) {
		int ret = ConfigReadStates.READ_STATE_INIT;
		if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_1_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_1_END;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_2_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_2_END;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_3_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_3_END;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_4_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_4_END;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_5_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_5_END;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_6_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_6_END;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_7_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_7_END;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_9_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_9_END;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_10_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_10_END;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_12_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_12_END;
		} else if (gattAttributes.equals(GattAttributes.CHARACTERISTIC_14_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_CHARACTERISTIC_14_END;
		} else if (gattAttributes.equals(GattAttributesTIOAD.CHARACTERISTIC_1_READ_UUID)) {
			ret = ConfigReadStates.READ_STATE_RESET_CHARACTERISTIC_1_END;
		}
		return ret;
	}

	/**
	 * 读取特征值的信息，包括解析数据
	 * 
	 * @param suuid
	 * @param cuuid
	 * @return
	 */
	private int readCharacteristic(UUID suuid, UUID cuuid) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		readState = getReadStateStart(cuuid.toString());
		final int readStateEnd = getReadStateEnd(cuuid.toString());
		// 读取数据
		result = readBle(suuid, cuuid);
		synchronized (lockedRead) {
			try {
				lockedRead.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
			}
		}
		if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT && readState == readStateEnd) {
			result = SKYBeaconCommunicateProtocol.solveCharacteristic(readDatas, readDatasLength, readStateEnd, skyBeacon);
		}
		return result;
	}

	/**
	 * 读取特征值的信息，包括解析数据
	 * 
	 * @param suuid
	 * @param cuuid
	 * @return
	 */
	private int writeCharacteristic(UUID suuid, UUID cuuid, byte[] writeDatas) {
		int result = ConfigInfoCode.ErrorCode.CODE_DEFAULT;
		writeState = false;
		// 读取数据
		result = writeBle(suuid, cuuid, writeDatas);
		synchronized (lockedWrite) {
			try {
				lockedWrite.wait(CONFIG_STEP_TIMEOUT_MILLISECOND);
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
			}
		}
		if (result == ConfigInfoCode.ErrorCode.CODE_DEFAULT) {
			if (!writeState) {
				result = ConfigInfoCode.ErrorCode.CODE_WRITE_CHARACTERISTIC_FAILED;
			}
		}
		return result;
	}

	@SuppressWarnings("unused")
	private void updateStates(int state, String msgStr) {
		if (isConfiging) {
			Message msg = new Message();
			msg.what = state;
			msg.obj = msgStr;
			handlerMessageListener.handlerMessage(msg);
		}
	}

	private class ImgHdr {
		short ver;
		short len;
		Character imgType;
		byte[] uid = new byte[4];
	}

	private class ProgInfo {
		int iBytes = 0; // Number of bytes programmed
		short iBlocks = 0; // Number of blocks programmed
		short nBlocks = 0; // Total number of blocks
		int iTimeElapsed = 0; // Time elapsed in milliseconds

		void reset() {
			iBytes = 0;
			iBlocks = 0;
			iTimeElapsed = 0;
			nBlocks = (short) (mFileImgHdr.len / (OAD_BLOCK_SIZE / HAL_FLASH_WORD_SIZE));
		}
	}

	private boolean loadFile(String filepath, boolean isAsset) {
		boolean fSuccess = true;

		// Load binary file
		try {
			// Read the file raw into a buffer
			InputStream stream;
			if (isAsset) {
				stream = context.getAssets().open(filepath);
			} else {
				File f = new File(filepath);
				stream = new FileInputStream(f);
			}
			stream.read(mFileBuffer, 0, mFileBuffer.length);
			stream.close();
		} catch (IOException e) {
			// Handle exceptions here
			// mLog.setText("File open failed: " + filepath + "\n");
			Log.e(TAG, "File open failed: " + filepath + "\n");
			return false;
		}

		// Show image info
		mFileImgHdr.ver = Conversion.buildUint16(mFileBuffer[5], mFileBuffer[4]);
		mFileImgHdr.len = Conversion.buildUint16(mFileBuffer[7], mFileBuffer[6]);
		mFileImgHdr.imgType = ((mFileImgHdr.ver & 1) == 1) ? 'B' : 'A';
		System.arraycopy(mFileBuffer, 8, mFileImgHdr.uid, 0, 4);
		// displayImageInfo(mFileImage, mFileImgHdr);

		// Verify image types
		// boolean ready = mFileImgHdr.imgType != mTargImgHdr.imgType;
		// int resid = ready ? R.style.dataStyle1 : R.style.dataStyle2;
		// mFileImage.setTextAppearance(this, resid);
		//
		// // Enable programming button only if image types differ
		// mBtnStart.setEnabled(ready);
		//
		// // Expected duration
		// displayStats();
		//
		// // Log
		// mLog.setText("Image " + mFileImgHdr.imgType + " selected.\n");
		// mLog.append(ready ? "Ready to program device!\n" :
		// "Incompatible image, select alternative!\n");
		//
		// updateGui();

		return fSuccess;
	}
}
