package com.skybeacon.sdk.locate;

import java.util.Date;

import android.bluetooth.BluetoothDevice;

import com.skybeacon.sdk.config.SKYBeaconEncrypt;
import com.skybeacon.sdk.config.SKYBeaconPower;
import com.skybeacon.sdk.config.SKYBeaconVersionUtil;
import com.skybeacon.sdk.decryption.DecryptProcess;
import com.skybeacon.sdk.entity.VersionAdaption;
import com.skybeacon.sdk.utils.DataConvertUtils;

/**
 * SeekcyBeacon的相关协议处理，静态函数
 * 
 * @author frank
 * 
 */
public class SKYBeaconScanProtocol {

	/**
	 * 格式化SKYBeaconScanRaw为SKYBeacon
	 * 
	 * @param skyBeaconScanRaw
	 * @return SKYBeacon 失败返回null
	 */
	public static SKYBeacon formatToSKYBeacon(SKYBeaconScanRaw skyBeaconScanRaw) {
		SKYBeacon skyBeacon = new SKYBeacon();

		skyBeacon = fromScanDataToSKYBeacon(skyBeaconScanRaw.getbluetoothDevice(), skyBeaconScanRaw.getRssi(), skyBeaconScanRaw.getScanRecord());

		return skyBeacon;
	}

	/**
	 * 格式化SKYBeaconScanRaw为SKYBeacon
	 * 
	 * @param skyBeaconScanRaw
	 * @return SKYBeacon 失败返回null
	 */
	public static SKYBeacon formatToSKYBeacon(SKYBeaconScanRaw skyBeaconScanRaw, SKYBeaconMultiIDs skyBeaconMultiIDs) {
		SKYBeacon skyBeacon = new SKYBeacon();

		skyBeacon = fromScanDataToSKYBeacon(skyBeaconScanRaw.getbluetoothDevice(), skyBeaconScanRaw.getRssi(), skyBeaconScanRaw.getScanRecord(), skyBeaconMultiIDs);

		return skyBeacon;
	}

	/**
	 * 格式化SKYBeaconScanRaw为SKYBeaconMultiIDs
	 * 
	 * @param skyBeaconScanRaw
	 * @return SKYBeaconMultiIDs 失败返回null
	 */
	public static SKYBeaconMultiIDs formatToSKYBeaconMultiIDs(SKYBeaconScanRaw skyBeaconScanRaw) {
		SKYBeaconMultiIDs skyBeaconMultiIDs = new SKYBeaconMultiIDs();

		skyBeaconMultiIDs = fromScanDataToSKYBeaconMultiIDs(skyBeaconScanRaw.getbluetoothDevice(), skyBeaconScanRaw.getRssi(), skyBeaconScanRaw.getScanRecord());

		return skyBeaconMultiIDs;
	}

	/**
	 * 根据MeasuredPower和rssi解算距离
	 * 
	 * @param measuredPower
	 * @param rssi
	 * @return
	 */
	public static float solveDistance(int measuredPower, int rssi) {
		float d = 0;

		if (rssi > (measuredPower + 20)) {
			d = (float) 0.1;
		} else if ((rssi > measuredPower) && (rssi <= (measuredPower + 20))) {
			d = (float) (0.002 * (rssi - measuredPower) * (rssi - measuredPower) - 0.0857 * (rssi - measuredPower) + 1); // 0.1至1米
		} else if ((rssi > (measuredPower - 15)) && (rssi <= measuredPower)) {
			d = (float) (0.0133 * (rssi - measuredPower) * (rssi - measuredPower) - 0.0667 * (rssi - measuredPower) + 1); // 1至5米
																															// 15分贝4米
		} else if ((rssi > (measuredPower - 25)) && (rssi <= (measuredPower - 15))) {
			d = (float) (0.033 * (rssi - measuredPower) * (rssi - measuredPower) + 0.8187 * (rssi - measuredPower) + 9.8626); // 5至10米
																																// 10分贝5米
		} else if ((rssi > (measuredPower - 33)) && (rssi <= (measuredPower - 25))) {
			d = (float) (0.05 * (rssi - measuredPower) * (rssi - measuredPower) + 2.15 * (rssi - measuredPower) + 32.5); // 10至16米
																															// 8分贝6米
		} else if ((rssi > (measuredPower - 39)) && (rssi <= (measuredPower - 33))) {
			d = (float) (0.0571 * (rssi - measuredPower) * (rssi - measuredPower) + 3.1143 * (rssi - measuredPower) + 56.5429); // 16至22米
																																// 6分贝6米
		} else if (rssi <= (measuredPower - 39)) {
			d = (float) (-1.3 * (rssi - measuredPower + 39) + 22); // 22米以上
																	// 1分贝1.3米
		}

		return d;
	}

	private static SKYBeacon fromScanDataToSKYBeacon(BluetoothDevice device, int rssi, byte[] scanData) {

		if (scanData == null) {
			return null;
		}

		int startByte = 2;
		boolean patternFound = false;
		int responseStartByte = 34;
		boolean responseFonud = false;
		while (startByte <= 5) {
			if (((int) scanData[startByte + 30] & 0xff) == 0x86 && ((int) scanData[startByte + 31] & 0xff) == 0x43) {
				responseStartByte = 34;
				responseFonud = true;
			}
			if (((int) scanData[startByte + 31] & 0xff) == 0x86 && ((int) scanData[startByte + 32] & 0xff) == 0x43) {
				responseStartByte = 35;
				responseFonud = true;
			}
			if (((int) scanData[startByte + 2] & 0xff) == 0x02 && ((int) scanData[startByte + 3] & 0xff) == 0x15) {
				// yes! This is an iBeacon
				patternFound = true;
				break;
			} else if (((int) scanData[startByte] & 0xff) == 0x2d && ((int) scanData[startByte + 1] & 0xff) == 0x24 && ((int) scanData[startByte + 2] & 0xff) == 0xbf
					&& ((int) scanData[startByte + 3] & 0xff) == 0x16) {
				SKYBeacon iBeacon = new SKYBeacon();
				return iBeacon;
			} else if (((int) scanData[startByte] & 0xff) == 0xad && ((int) scanData[startByte + 1] & 0xff) == 0x77 && ((int) scanData[startByte + 2] & 0xff) == 0x00
					&& ((int) scanData[startByte + 3] & 0xff) == 0xc6) {
				SKYBeacon iBeacon = new SKYBeacon();
				return iBeacon;
			}
			startByte++;
		}

		if (patternFound == false) {
			// This is not an iBeacon
			return null;
		}

		SKYBeacon ibeacon = new SKYBeacon();
		if (responseFonud) {
			byte[] decryptScanDatas = new byte[27];
			decryptScanDatas = SKYBeaconEncrypt.decryptResponseDatas(scanData, responseStartByte);
			ibeacon.setHardwareVersion(decryptScanDatas[0] & 0x00FF);
			ibeacon.setFirmwareVersionMajor(decryptScanDatas[1] & 0x00FF);
			ibeacon.setFirmwareVersionMinor(decryptScanDatas[2] & 0x00FF);
			ibeacon.setBattery(decryptScanDatas[3]);
			ibeacon.setTemperature(decryptScanDatas[4]);
			if (decryptScanDatas[5] == 0x01) {
				ibeacon.setLocked(1);
			} else {
				ibeacon.setLocked(0);
			}
			ibeacon.setTxpower(SKYBeaconPower.getPower((int) decryptScanDatas[21]));
			ibeacon.setIntervalMillisecond(Integer.valueOf((decryptScanDatas[22] & 0x00FF)) * 10);
		}
		ibeacon.setTimestampMillisecond(DataConvertUtils.getCurrentTimestampMillisecond());
		ibeacon.setMajor((scanData[startByte + 20] & 0xff) * 0x100 + (scanData[startByte + 21] & 0xff));
		ibeacon.setMinor((scanData[startByte + 22] & 0xff) * 0x100 + (scanData[startByte + 23] & 0xff));
		ibeacon.setMeasuredPower((int) scanData[startByte + 24]);
		ibeacon.setRssi(rssi);

		// AirLocate:
		// 02 01 1a 1a ff 4c 00 02 15 # Apple's fixed iBeacon advertising prefix
		// e2 c5 6d b5 df fb 48 d2 b0 60 d0 f5 a7 10 96 e0 # iBeacon profile
		// uuid
		// 00 00 # major
		// 00 00 # minor
		// c5 # The 2's complement of the calibrated Tx Power
		byte[] proximityUuidBytes = new byte[16];
		System.arraycopy(scanData, startByte + 4, proximityUuidBytes, 0, 16);
		String hexString = DataConvertUtils.bytesToHexString(proximityUuidBytes);
		StringBuilder sb = new StringBuilder();
		sb.append(hexString.substring(0, 8));
		sb.append("-");
		sb.append(hexString.substring(8, 12));
		sb.append("-");
		sb.append(hexString.substring(12, 16));
		sb.append("-");
		sb.append(hexString.substring(16, 20));
		sb.append("-");
		sb.append(hexString.substring(20, 32));
		ibeacon.setProximityUUID(sb.toString());

		if (device != null) {
			ibeacon.setDeviceAddress(device.getAddress());
			ibeacon.setDeviceName(device.getName());
			ibeacon.setDistance(solveDistance(ibeacon.getMeasuredPower(), ibeacon.getRssi()));
		}

		return ibeacon;
	}

	private static SKYBeacon fromScanDataToSKYBeacon(BluetoothDevice device, int rssi, byte[] scanData, SKYBeaconMultiIDs beaconMultiIDs) {

		if (scanData == null) {
			return null;
		}

		// TODO
		// scanData =
		// DataConvertUtils.hexStringToByte("0201061AFF4C000215E2C56DB5DFFB48D2B060D0F5A71096E000FF00FFCD1E1686436E7F7B7A227F2E1674D6182EB38EAF577A857A85B77A521DA18F7A");
		int startByte = 2;
		boolean patternFound = false;
		int responseStartByte = 34;
		boolean responseFonud = false;
		SKYBeacon.SKYDeviceType deviceType = SKYBeacon.SKYDeviceType.DEVICE_TYPE_IBEACON;
		while (startByte <= 5) {
			if (((int) scanData[startByte + 30] & 0xff) == 0x86 && ((int) scanData[startByte + 31] & 0xff) == 0x43) {
				responseStartByte = 34;
				responseFonud = true;
			}
			if (((int) scanData[startByte + 31] & 0xff) == 0x86 && ((int) scanData[startByte + 32] & 0xff) == 0x43) {
				responseStartByte = 35;
				responseFonud = true;
			}
			if (((int) scanData[startByte + 30] & 0xff) == 0x86 && ((int) scanData[startByte + 31] & 0xff) == 0x44) {
				responseStartByte = 34;
				responseFonud = true;
			}
			if (((int) scanData[startByte + 2] & 0xff) == 0x02 && ((int) scanData[startByte + 3] & 0xff) == 0x15) {
				// yes! This is an iBeacon
				patternFound = true;
				break;
			} else if (((int) scanData[startByte] & 0xff) == 0x2d && ((int) scanData[startByte + 1] & 0xff) == 0x24 && ((int) scanData[startByte + 2] & 0xff) == 0xbf
					&& ((int) scanData[startByte + 3] & 0xff) == 0x16) {
				SKYBeacon iBeacon = new SKYBeacon();
				return iBeacon;
			} else if (((int) scanData[startByte] & 0xff) == 0xad && ((int) scanData[startByte + 1] & 0xff) == 0x77 && ((int) scanData[startByte + 2] & 0xff) == 0x00
					&& ((int) scanData[startByte + 3] & 0xff) == 0xc6) {
				SKYBeacon iBeacon = new SKYBeacon();
				return iBeacon;
			} else if (((int) scanData[startByte + 2] & 0xff) == 0x86 && ((int) scanData[startByte + 3] & 0xff) == 0x44) {
				patternFound = true;
				deviceType = SKYBeacon.SKYDeviceType.DEVICE_TYPE_FOOT_RING;
				break;
			}
			startByte++;
		}

		if (patternFound == false) {
			// This is not an iBeacon
			return null;
		}

		SKYBeacon ibeacon = new SKYBeacon();
		if (deviceType == SKYBeacon.SKYDeviceType.DEVICE_TYPE_IBEACON) {
			if (responseFonud) {
				// TODO
				// if (device.getAddress().equals("19:18:FC:07:DB:96")) {
				// String tmp = "";
				// String tmp1 = tmp;
				// } else {
				// return null;
				// }
				byte[] decryptScanDatas = new byte[27];
				decryptScanDatas = SKYBeaconEncrypt.decryptResponseDatas(scanData, responseStartByte);
				ibeacon.setSeekcyBeacon(1);
				ibeacon.setHardwareVersion(decryptScanDatas[0] & 0x00FF);
				ibeacon.setFirmwareVersionMajor(decryptScanDatas[1] & 0x00FF);
				ibeacon.setFirmwareVersionMinor(decryptScanDatas[2] & 0x00FF);
				ibeacon.setDeviceModel(VersionAdaption.getProductName(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor()));
				// 判断response协议（是否支持OAD）
				if ((decryptScanDatas[5] & 0x04) > 0x00) {
					ibeacon.setIsOAD(1);
				} else {
					ibeacon.setIsOAD(0);
				}
				// 判断response协议（是否为OAD Mode下）
				if (ibeacon.getIsOAD() == 1) {
					if ((decryptScanDatas[5] & 0x08) > 0x00) {
						ibeacon.setOADMode(false);
					} else {
						ibeacon.setOADMode(true);
					}
				}
				if (!ibeacon.isOADMode()) {
					ibeacon.setBattery(decryptScanDatas[3]);
					if (VersionAdaption.isTempteratureSensorContain(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
						ibeacon.setTemperature(decryptScanDatas[4]);
					}
					if (VersionAdaption.isLightSensorContain(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
						ibeacon.setLightValue((decryptScanDatas[24] & 0x00FF) * 4);
					}
					ibeacon.setTxpower(SKYBeaconPower.getPower((int) decryptScanDatas[21]));
					if (SKYBeaconVersionUtil.isIntervalFiveSecondProtocol(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
						int tmpInterval = Integer.valueOf((decryptScanDatas[22] & 0x00FF));
						if (tmpInterval > 100) {
							ibeacon.setIntervalMillisecond((tmpInterval - 100) * 50 + 1000);
						} else {
							ibeacon.setIntervalMillisecond(tmpInterval * 10);
						}
					} else {
						ibeacon.setIntervalMillisecond(Integer.valueOf((decryptScanDatas[22] & 0x00FF)) * 10);
					}
				}
				if ((decryptScanDatas[5] & 0x02) > 0) {
					ibeacon.setEncrypted(1);
				} else {
					ibeacon.setEncrypted(0);
				}

				if ((decryptScanDatas[5] & 0x01) > 0x00) {
					ibeacon.setLocked(1);
				} else {
					ibeacon.setLocked(0);
				}
				// 根据版本号分支处理
				if (VersionAdaption.isMultiIDsVersion(decryptScanDatas[0], decryptScanDatas[1])) {
					beaconMultiIDs.setTimestampMillisecond(DataConvertUtils.getCurrentTimestampMillisecond());
					beaconMultiIDs.setHardwareVersion(decryptScanDatas[0] & 0x00FF);
					beaconMultiIDs.setFirmwareVersionMajor(decryptScanDatas[1] & 0x00FF);
					beaconMultiIDs.setFirmwareVersionMinor(decryptScanDatas[2] & 0x00FF);
					if (!ibeacon.isOADMode()) {
						beaconMultiIDs.setBattery(decryptScanDatas[3]);
						if (VersionAdaption.isTempteratureSensorContain(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
							beaconMultiIDs.setTemperature(decryptScanDatas[4]);
						}
						if (VersionAdaption.isLightSensorContain(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
							beaconMultiIDs.setLightValue((decryptScanDatas[24] & 0x00FF) * 4);
						}
						if (SKYBeaconVersionUtil.isIntervalFiveSecondProtocol(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
							int tmpInterval = Integer.valueOf((decryptScanDatas[22] & 0x00FF));
							if (tmpInterval > 100) {
								beaconMultiIDs.setIntervalMillisecond((tmpInterval - 100) * 50 + 1000);
							} else {
								beaconMultiIDs.setIntervalMillisecond(tmpInterval * 10);
							}
						} else {
							beaconMultiIDs.setIntervalMillisecond(Integer.valueOf((decryptScanDatas[22] & 0x00FF)) * 10);
						}
					}
					beaconMultiIDs.setSeekcyBeacon(1);
					// // mac process
					// byte[] tmpMac = new byte[] { decryptScanDatas[6],
					// decryptScanDatas[7], decryptScanDatas[8],
					// decryptScanDatas[9], decryptScanDatas[10],
					// decryptScanDatas[11] };
					// if (ibeacon.isEncrypted() == 1) {
					// byte[] tmpMac2 = new byte[] { decryptScanDatas[9],
					// decryptScanDatas[10] };
					// byte[] tmp1 = new byte[16];
					// byte[] tmp2 = new byte[2];
					// DecryptProcess.getUuidMajorMinor(tmp1, tmpMac2, tmp2);
					// tmpMac[3] = (byte) ((tmpMac2[0] & 0xFF) |
					// (decryptScanDatas[23] & 0xC0));
					// tmpMac[4] = tmpMac2[1];
					// }
					// beaconMultiIDs.setDeviceAddress(DataConvertUtils
					// .bytesToMacString(tmpMac));

					if ((decryptScanDatas[5] & 0x01) > 0x00) {
						beaconMultiIDs.setLocked(1);
					} else {
						beaconMultiIDs.setLocked(0);
					}
				}

				if (VersionAdaption.isS4LRRTC(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
					Long lt = Long.valueOf(DataConvertUtils.byte4ToInt(new byte[] { decryptScanDatas[4], decryptScanDatas[23], decryptScanDatas[24], decryptScanDatas[25] }));
					ibeacon.setiBeaconDate(lt * 1000);
				}
			}
			ibeacon.setTimestampMillisecond(DataConvertUtils.getCurrentTimestampMillisecond());
			if (ibeacon.isEncrypted() != 1) {
				ibeacon.setMajor((((int) (scanData[startByte + 20] & 0x00ff)) << 8) + (int) (scanData[startByte + 21] & 0x00ff));
				ibeacon.setMinor((((int) (scanData[startByte + 22] & 0x00ff)) << 8) + (scanData[startByte + 23] & 0x00ff));
			} else {
				// TODO
				// 5 35680 42479 35680 42479 1.45581E+12 864 24672
				// byte[] majorTmp1 = new byte[] { (byte) 0x8B, 0x60 };
				// byte[] minorTmp1 = new byte[] { (byte) 0xA5, (byte) 0xEF };
				// byte[] majorTmp1 = new byte[] { scanData[startByte + 20],
				// scanData[startByte + 21] };
				// byte[] minorTmp1 = new byte[] { scanData[startByte + 22],
				// scanData[startByte + 23] };

				// byte[] majorTmp = new byte[] { (byte) 0x8B, 0x60 };
				byte[] majorTmp = new byte[] { scanData[startByte + 20], scanData[startByte + 21] };
				byte[] tmp1 = new byte[16];
				byte[] tmp2 = new byte[2];
				if (SKYBeaconVersionUtil.isEncryptedVersion2(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
					// byte[] minorTmp = new byte[] { (byte) 0xA5, (byte) 0xEF
					// };
					byte[] minorTmp = new byte[] { scanData[startByte + 22], scanData[startByte + 23] };
					DecryptProcess.getUuidMajorMinorV2(tmp1, majorTmp, minorTmp);
					ibeacon.setMajor((((int) (majorTmp[0] & 0x00ff)) << 8) + (int) (majorTmp[1] & 0x00ff));
					ibeacon.setMinor((((int) (minorTmp[0] & 0x00ff)) << 8) + (int) (minorTmp[1] & 0x00ff));

					// // TODO test
					// String logTmp = "";
					// int majorIntTmp = (((int) (majorTmp1[0] & 0x00ff)) << 8)
					// +
					// (int) (majorTmp1[1] & 0x00ff);
					// int minotIntTmp = (((int) (minorTmp1[0] & 0x00ff)) << 8)
					// +
					// (int) (minorTmp1[1] & 0x00ff);
					// logTmp += String.valueOf(((int) (majorTmp1[0] & 0x7c) >>
					// 1) +
					// ((int) (minorTmp1[0] & 0x80) >> 7));
					// logTmp += "\t" + String.valueOf(majorIntTmp);
					// logTmp += "\t" + String.valueOf(minotIntTmp);
					// logTmp += "\t" + String.valueOf(ibeacon.getMajor());
					// logTmp += "\t" + String.valueOf(ibeacon.getMinor());
					// logTmp += "\t" +
					// String.valueOf(DataConvertUtils.getCurrentTimestampMillisecond());
					// logTmp += "\t" + String.valueOf((((int)
					// (scanData[startByte +
					// 16] & 0x00ff)) << 8) + (int) (scanData[startByte + 17] &
					// 0x00ff));
					// logTmp += "\t" + String.valueOf((((int)
					// (scanData[startByte +
					// 18] & 0x00ff)) << 8) + (int) (scanData[startByte + 19] &
					// 0x00ff)) + "\r\n";
					// try {
					// // if (ibeacon.getHardwareVersion() == 5 &&
					// // ibeacon.getFirmwareVersionMajor() == 3) {
					// if (device.getAddress().equals("19:18:FD:00:FF:FF")) {
					// FileAdapter.writeSDFileAppend(PublicStaticValues.FILE_ROOT
					// +
					// PublicStaticValues.FILE_LOGS_DERECTORY, "encryptLog.txt",
					// logTmp);
					// }
					// // }
					// } catch (IOException e) {
					// // TODO Auto-generated catch block
					// e.printStackTrace();
					// }
				} else if (SKYBeaconVersionUtil.isEncryptedVersion3(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
					if (SKYBeaconManager.getInstance().isDecryptScan()) {
						byte[] minorTmp = new byte[] { scanData[startByte + 22], scanData[startByte + 23] };
						DecryptProcess.getUuidMajorMinorV3(tmp1, majorTmp, minorTmp);
						ibeacon.setMajor((((int) (majorTmp[0] & 0x00ff)) << 8) + (int) (majorTmp[1] & 0x00ff));
						ibeacon.setMinor((((int) (minorTmp[0] & 0x00ff)) << 8) + (int) (minorTmp[1] & 0x00ff));

						// mac process
						byte[] macTmp = DataConvertUtils.hexStringToByte(device.getAddress().replace(":", ""));
						byte[] macTmp1 = new byte[] { scanData[startByte + 20], scanData[startByte + 21] };
						byte[] macTmp2 = new byte[] { macTmp[4], macTmp[5] };
						DecryptProcess.getUuidMajorMinorV3(tmp1, macTmp1, macTmp2);
						macTmp[4] = macTmp2[0];
						macTmp[5] = macTmp2[1];
						ibeacon.setDeviceAddressDecrypt(DataConvertUtils.bytesToMacString(macTmp));
					} else {
						ibeacon.setMajor((((int) (scanData[startByte + 20] & 0x00ff)) << 8) + (int) (scanData[startByte + 21] & 0x00ff));
						ibeacon.setMinor((((int) (scanData[startByte + 22] & 0x00ff)) << 8) + (scanData[startByte + 23] & 0x00ff));
					}
				} else {
					DecryptProcess.getUuidMajorMinor(tmp1, majorTmp, tmp2);
					ibeacon.setMajor((majorTmp[0] & 0xff) * 0x100 + (majorTmp[1] & 0xff));
					ibeacon.setMinor((scanData[startByte + 22] & 0xff) * 0x100 + (scanData[startByte + 23] & 0xff));
				}
			}
			ibeacon.setMeasuredPower((int) scanData[startByte + 24]);
			ibeacon.setRssi(rssi);

			// AirLocate:
			// 02 01 1a 1a ff 4c 00 02 15 # Apple's fixed iBeacon advertising
			// prefix
			// e2 c5 6d b5 df fb 48 d2 b0 60 d0 f5 a7 10 96 e0 # iBeacon profile
			// uuid
			// 00 00 # major
			// 00 00 # minor
			// c5 # The 2's complement of the calibrated Tx Power
			byte[] proximityUuidBytes = new byte[16];
			System.arraycopy(scanData, startByte + 4, proximityUuidBytes, 0, 16);
			String hexString = DataConvertUtils.bytesToHexString(proximityUuidBytes);
			StringBuilder sb = new StringBuilder();
			sb.append(hexString.substring(0, 8));
			sb.append("-");
			sb.append(hexString.substring(8, 12));
			sb.append("-");
			sb.append(hexString.substring(12, 16));
			sb.append("-");
			sb.append(hexString.substring(16, 20));
			sb.append("-");
			sb.append(hexString.substring(20, 32));
			ibeacon.setProximityUUID(sb.toString());

			if (device != null) {
				ibeacon.setDeviceAddress(device.getAddress());
				ibeacon.setDeviceName(device.getName());
				// 在add的时候操作添加distance
				ibeacon.setDistance(solveDistance(ibeacon.getMeasuredPower(), ibeacon.getRssi()));
			}

			// 对多id的iBeacon做特殊的处理
			byte[] deviceAddressTmp = DataConvertUtils.macStringToBytes(device.getAddress());

			if (deviceAddressTmp.length == 6) {
				if (deviceAddressTmp[0] == (byte) 0x19 && deviceAddressTmp[1] == (byte) 0x18 && ((deviceAddressTmp[3] & (byte) 0xC0) != 0) && (beaconMultiIDs.isSeekcyBeacon() != 1)) {
					deviceAddressTmp[3] = (byte) (deviceAddressTmp[3] & 0x3F);
					beaconMultiIDs.setSeekcyBeacon(1);
					beaconMultiIDs.setDeviceAddress(DataConvertUtils.bytesToMacString(deviceAddressTmp));
					beaconMultiIDs.setTimestampMillisecond(DataConvertUtils.getCurrentTimestampMillisecond());
				} else if (deviceAddressTmp[0] == (byte) 0x19 && deviceAddressTmp[1] == (byte) 0x18) {
					deviceAddressTmp[3] = (byte) (deviceAddressTmp[3] & 0x3F);
					beaconMultiIDs.setDeviceAddress(DataConvertUtils.bytesToMacString(deviceAddressTmp));
				}
			}
		} else if (deviceType == SKYBeacon.SKYDeviceType.DEVICE_TYPE_FOOT_RING) {
			if (responseFonud) {
				byte[] decryptScanDatas = new byte[27];
				decryptScanDatas = SKYBeaconEncrypt.decryptResponseDatas(scanData, responseStartByte);
				ibeacon.setSeekcyBeacon(1);
				ibeacon.setDeviceType(SKYBeacon.SKYDeviceType.DEVICE_TYPE_FOOT_RING);
				ibeacon.setHardwareVersion(decryptScanDatas[0] & 0x00FF);
				ibeacon.setFirmwareVersionMajor(decryptScanDatas[1] & 0x00FF);
				ibeacon.setFirmwareVersionMinor(decryptScanDatas[2] & 0x00FF);
				ibeacon.setBattery(decryptScanDatas[3]);
				if (VersionAdaption.isTempteratureSensorContain(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
					ibeacon.setTemperature(decryptScanDatas[4]);
				}
				if (VersionAdaption.isLightSensorContain(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
					ibeacon.setLightValue((decryptScanDatas[24] & 0x00FF) * 4);
				}
				ibeacon.setTxpower(SKYBeaconPower.getPower((int) decryptScanDatas[21]));
				if (SKYBeaconVersionUtil.isIntervalFiveSecondProtocol(ibeacon.getHardwareVersion(), ibeacon.getFirmwareVersionMajor(), ibeacon.getFirmwareVersionMinor())) {
					int tmpInterval = Integer.valueOf((decryptScanDatas[22] & 0x00FF));
					if (tmpInterval > 100) {
						ibeacon.setIntervalMillisecond((tmpInterval - 100) * 50 + 1000);
					} else {
						ibeacon.setIntervalMillisecond(tmpInterval * 10);
					}
				} else {
					ibeacon.setIntervalMillisecond(Integer.valueOf((decryptScanDatas[22] & 0x00FF)) * 10);
				}

				if ((decryptScanDatas[5] & 0x01) > 0x00) {
					ibeacon.setLocked(1);
				} else {
					ibeacon.setLocked(0);
				}
				// touch test interval
				ibeacon.setTouchTestInterval((decryptScanDatas[23] & 0x0070) >> 4);
				ibeacon.setAlarmBroadcastCounter(decryptScanDatas[23] & 0x000F);
				ibeacon.setBroadcastPassageway(getBroadcastPassageway((decryptScanDatas[25] & 0x000C) >> 2));
				ibeacon.setIsOAD(decryptScanDatas[25] & 0x0001);
			}
			ibeacon.setTimestampMillisecond(DataConvertUtils.getCurrentTimestampMillisecond());
			ibeacon.setRssi(rssi);
			ibeacon.setHardwareVersion(scanData[startByte + 2] & 0x00FF);
			ibeacon.setFirmwareVersionMajor(scanData[startByte + 3] & 0x00FF);
			ibeacon.setFirmwareVersionMinor(scanData[startByte + 4] & 0x00FF);
			ibeacon.setEventType(scanData[startByte + 5] & 0x00FF);
			ibeacon.setBattery(scanData[startByte + 6] & 0x00FF);
			ibeacon.setTriggerNum(DataConvertUtils.byte2ToInt(new byte[] { scanData[startByte + 7], scanData[startByte + 8] }));
			ibeacon.setAntennaNum(scanData[startByte + 9] & 0x00FF);
			int tmpInterval = Integer.valueOf((scanData[startByte + 10] & 0x00FF));
			if (tmpInterval > 100) {
				ibeacon.setIntervalMillisecond((tmpInterval - 100) * 50 + 1000);
			} else {
				ibeacon.setIntervalMillisecond(tmpInterval * 10);
			}
			ibeacon.setTriggerCounter(DataConvertUtils.byte4ToInt(new byte[] { scanData[startByte + 11], scanData[startByte + 12] }));
			ibeacon.setBroadcastCounter(scanData[startByte + 13] & 0x00FF);
			ibeacon.setTxpower(SKYBeaconPower.getPower((int) scanData[startByte + 14]));
			ibeacon.setAntennaCounter(DataConvertUtils.byte2ToInt(new byte[] { scanData[startByte + 15], scanData[startByte + 16] }));
			ibeacon.setBroadcastPassageway(getBroadcastPassageway(scanData[startByte + 17] & 0x00FF));
			ibeacon.setTamperAlarmCounter(scanData[startByte + 18] & 0x00FF);
			ibeacon.setTouchAlarmCounter(scanData[startByte + 19] & 0x00FF);
			ibeacon.setAlarmBroadcastIntervalMillisecond((scanData[startByte + 20] & 0x00FF) * 10);
			if (device != null) {
				ibeacon.setDeviceAddress(device.getAddress());
				ibeacon.setDeviceName(device.getName());
			}
		}
		// TODO
		// ibeacon.setDeviceAddress("54:6C:0E:AC:62:54");

		return ibeacon;
	}

	private static int getBroadcastPassageway(int raw) {
		int result = 0;
		if (raw == 0) {
			result = 37;
		} else if (raw == 1) {
			result = 38;
		} else if (raw == 2) {
			result = 39;
		}
		return result;
	}

	private static SKYBeaconMultiIDs fromScanDataToSKYBeaconMultiIDs(BluetoothDevice device, int rssi, byte[] scanData) {

		if (scanData == null) {
			return null;
		}

		int startByte = 2;
		boolean patternFound = false;
		int responseStartByte = 34;
		boolean responseFonud = false;
		while (startByte <= 5) {
			if (((int) scanData[startByte + 30] & 0xff) == 0x86 && ((int) scanData[startByte + 31] & 0xff) == 0x43) {
				responseStartByte = 34;
				responseFonud = true;
			}
			if (((int) scanData[startByte + 31] & 0xff) == 0x86 && ((int) scanData[startByte + 32] & 0xff) == 0x43) {
				responseStartByte = 35;
				responseFonud = true;
			}
			if (((int) scanData[startByte + 2] & 0xff) == 0x02 && ((int) scanData[startByte + 3] & 0xff) == 0x15) {
				// yes! This is an iBeacon
				patternFound = true;
				break;
			} else if (((int) scanData[startByte] & 0xff) == 0x2d && ((int) scanData[startByte + 1] & 0xff) == 0x24 && ((int) scanData[startByte + 2] & 0xff) == 0xbf
					&& ((int) scanData[startByte + 3] & 0xff) == 0x16) {
				SKYBeaconMultiIDs ibeacon = new SKYBeaconMultiIDs();
				return ibeacon;
			} else if (((int) scanData[startByte] & 0xff) == 0xad && ((int) scanData[startByte + 1] & 0xff) == 0x77 && ((int) scanData[startByte + 2] & 0xff) == 0x00
					&& ((int) scanData[startByte + 3] & 0xff) == 0xc6) {
				SKYBeaconMultiIDs ibeacon = new SKYBeaconMultiIDs();
				return ibeacon;
			}
			startByte++;
		}

		if (patternFound == false) {
			// This is not an iBeacon
			return null;
		}

		SKYBeaconMultiIDs ibeacons = new SKYBeaconMultiIDs();
		SKYBeacon ibeacon = new SKYBeacon();
		if (responseFonud) {
			byte[] decryptScanDatas = new byte[27];
			decryptScanDatas = SKYBeaconEncrypt.decryptResponseDatas(scanData, responseStartByte);
			ibeacon.setHardwareVersion(decryptScanDatas[0] & 0x00FF);
			ibeacon.setFirmwareVersionMajor(decryptScanDatas[1] & 0x00FF);
			ibeacon.setFirmwareVersionMinor(decryptScanDatas[2] & 0x00FF);
			ibeacons.setBattery(decryptScanDatas[3]);
			ibeacons.setTemperature(decryptScanDatas[4]);
			if ((decryptScanDatas[5] & 0x04) > 0) {
				ibeacon.setEncrypted(1);
			} else {
				ibeacon.setEncrypted(0);
			}
			// mac process TODO
			/**
			 * mac地址处理部分
			 */

			if ((decryptScanDatas[5] & 0x01) > 0) {
				ibeacons.setLocked(1);
				ibeacon.setLocked(1);
			} else {
				ibeacons.setLocked(0);
				ibeacon.setLocked(0);
			}
			ibeacon.setTxpower(SKYBeaconPower.getPower((int) decryptScanDatas[21]));
			ibeacon.setIntervalMillisecond(Integer.valueOf((decryptScanDatas[22] & 0x00FF)) * 10);
			ibeacons.setIntervalMillisecond(Integer.valueOf((decryptScanDatas[22] & 0x00FF)) * 10);
		}
		ibeacon.setTimestampMillisecond(DataConvertUtils.getCurrentTimestampMillisecond());
		ibeacons.setTimestampMillisecond(DataConvertUtils.getCurrentTimestampMillisecond());
		if (ibeacon.isEncrypted() != 1) {
			ibeacon.setMajor((scanData[startByte + 20] & 0xff) * 0x100 + (scanData[startByte + 21] & 0xff));
		} else {
			byte[] majorTmp = new byte[] { scanData[startByte + 20], scanData[startByte + 21] };
			byte[] tmp1 = new byte[16];
			byte[] tmp2 = new byte[2];
			DecryptProcess.getUuidMajorMinor(tmp1, majorTmp, tmp2);
			ibeacon.setMajor((majorTmp[0] & 0xff) * 0x100 + (majorTmp[1] & 0xff));
		}
		ibeacon.setMinor((scanData[startByte + 22] & 0xff) * 0x100 + (scanData[startByte + 23] & 0xff));
		ibeacon.setMeasuredPower((int) scanData[startByte + 24]);
		ibeacon.setRssi(rssi);

		// AirLocate:
		// 02 01 1a 1a ff 4c 00 02 15 # Apple's fixed iBeacon advertising prefix
		// e2 c5 6d b5 df fb 48 d2 b0 60 d0 f5 a7 10 96 e0 # iBeacon profile
		// uuid
		// 00 00 # major
		// 00 00 # minor
		// c5 # The 2's complement of the calibrated Tx Power
		byte[] proximityUuidBytes = new byte[16];
		System.arraycopy(scanData, startByte + 4, proximityUuidBytes, 0, 16);
		String hexString = DataConvertUtils.bytesToHexString(proximityUuidBytes);
		StringBuilder sb = new StringBuilder();
		sb.append(hexString.substring(0, 8));
		sb.append("-");
		sb.append(hexString.substring(8, 12));
		sb.append("-");
		sb.append(hexString.substring(12, 16));
		sb.append("-");
		sb.append(hexString.substring(16, 20));
		sb.append("-");
		sb.append(hexString.substring(20, 32));
		ibeacon.setProximityUUID(sb.toString());

		if (device != null) {
			ibeacon.setDeviceAddress(device.getAddress());
			ibeacon.setDeviceName(device.getName());
			ibeacon.setDistance(solveDistance(ibeacon.getMeasuredPower(), ibeacon.getRssi()));
		}
		ibeacons.addBeaconList(ibeacon);

		return ibeacons;
	}
}
